//
//  i2c_lcd.h
//
//  I2C interface type Character LCD module Control
//		LCD controler: ST7032i
//		LCD module   : Low voltage I2C LCD module SB1602B / Strowbery Linux Co.,Ltd.
//  Copyright (C) 2009 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.06.28 v0.0  First cording
//	2009.07.02 v0.0  Rewrite & 1st release
//	2009.07.05 v0.0  Compile error Bug fix
//

#include <m8c.h>        // part specific constants and macros

//
// Configure
//
// Need modify real interface
#include "I2Cm_1.h"
#define I2C_LCD_START()			I2Cm_1_Start()
#define I2C_LCD_STOP()			I2Cm_1_Stop()
#define I2C_LCD_WRITE(a,b,c,d)	I2Cm_1_bWriteBytes(a,b,c,d)

//
// API
//
// standard / compatible
extern void I2C_LCD_Delay50uTimes(BYTE bTimes);
extern void I2C_LCD_Delay1mTimes(BYTE bTimes);
extern void I2C_LCD_Start();
extern void I2C_LCD_Stop();
extern void I2C_LCD_Init();
extern void I2C_LCD_Position(BYTE bRow, BYTE bCol);
#define I2C_LCD_Control(c)   I2C_LCD_SendCmdData(I2C_LCD_CB_INST, c)
#define I2C_LCD_WriteData(d) I2C_LCD_SendCmdData(I2C_LCD_CB_DATA, d)
#define I2C_LCD_PrCString(s) printf(s)
#define I2C_LCD_PrString(s)  printf(s)
#define I2C_LCD_PrHexByte(b) printf("%02x", b)
#define I2C_LCD_PrHexInt(i)  printf("%04x", i)
// special
extern void I2C_LCD_Contrast(BYTE bCont);
extern void I2C_LCD_Icon(BYTE bAddr, BYTE bFlag);
extern void I2C_LCD_SendCmdData(BYTE cmd, char data);

//
// Control command
//
// PSoC Designer LCD module compatible define
#define I2C_LCD_DISP_ON                       0x0C
#define I2C_LCD_DISP_OFF                      0x08
#define I2C_LCD_DISP_BLANK                    0x0A
#define I2C_LCD_DISP_CLEAR_HOME               0x01
#define I2C_LCD_CURSOR_ON                     0x0E
#define I2C_LCD_CURSOR_OFF                    0x0C
#define I2C_LCD_CURSOR_WINK                   0x0D
#define I2C_LCD_CURSOR_BLINK                  0x0F
#define I2C_LCD_CURSOR_SH_LEFT                0x10
#define I2C_LCD_CURSOR_SH_RIGHT               0x14
#define I2C_LCD_CURSOR_HOME                   0x02
#define I2C_LCD_CURSOR_LEFT                   0x04
#define I2C_LCD_CURSOR_RIGHT                  0x06
#define I2C_LCD_PORT_MASK                     0x7F

// ST7032i I2C stream
#define I2C_LCD_ADDR		0x3e		// I2C slave address
#define I2C_LCD_CB_CONT		0x80		// I2C Command byte -> Continue stream
#define I2C_LCD_CB_INST		0x00		// I2C Command byte -> next byte is instruction
#define I2C_LCD_CB_DATA		0x40		// I2C Command byte -> next byte is data

// Instruction: "Function Set"
#define I2C_LCD_INST_FUNC 0x20			// ID
#define I2C_LCD_DL_8BIT   0x10			// DL: Interface data length
#define I2C_LCD_DL_4BIT   0x00			//
#define I2C_LCD_N_2LINE   0x08			// N: Display line number  1=2-line 0=1-line
#define I2C_LCD_N_1LINE   0x00			//
#define I2C_LCD_DH_DOUBLE 0x04			// DH: Double height font type
#define I2C_LCD_DH_NORMAL 0x00			//
#define I2C_LCD_IS_EXTEND 0x01			// IS: normal/extension instruction select
#define I2C_LCD_IS_NORMAL 0x00			//

//	Instruction: <extention mode> "Bias selection / Internal OSC frequency adjust"
#define I2C_LCD_INST_BIAS 0x10			// ID
#define I2C_LCD_BS_4      0x08			// BS: bias selection  1/4
#define I2C_LCD_BS_5      0x00			//                     1/5
#define I2C_LCD_OSC_122HZ 0x00			// Internal OSC frequency adjust
#define I2C_LCD_OSC_131HZ 0x01			// 
#define I2C_LCD_OSC_144HZ 0x02			// 
#define I2C_LCD_OSC_161HZ 0x03			// 
#define I2C_LCD_OSC_183HZ 0x04			// 
#define I2C_LCD_OSC_221HZ 0x05			// 
#define I2C_LCD_OSC_274HZ 0x06			// 
#define I2C_LCD_OSC_347HZ 0x07			// 
#define I2C_LCD_OSC_120HZ 0x00			// Vdd=5.0V
#define I2C_LCD_OSC_133HZ 0x01			// Vdd=5.0V
#define I2C_LCD_OSC_149HZ 0x02			// Vdd=5.0V
#define I2C_LCD_OSC_167HZ 0x03			// Vdd=5.0V
#define I2C_LCD_OSC_192HZ 0x04			// Vdd=5.0V
#define I2C_LCD_OSC_227HZ 0x05			// Vdd=5.0V
#define I2C_LCD_OSC_277HZ 0x06			// Vdd=5.0V

//	Instruction: <extention mode> "Set ICON address"
#define I2C_LCD_INST_ICON 0x40			// ID
#define I2C_LCD_ICON_ADDR 0x0f			// adress mask bit, next data is icon patturn bit

//	Instruction: <extention mode> "Power/ICON control/Contrast set"
#define I2C_LCD_INST_POW  0x50			// ID
#define I2C_LCD_I_ON      0x08			// Ion: set ICON display
#define I2C_LCD_I_OFF     0x00			// 
#define I2C_LCD_B_ON      0x04			// Bon: switch booster circuit
#define I2C_LCD_B_OFF     0x00			// 
#define I2C_LCD_C_HIGH    0x03			// C5~4: Contrast value high nibble

//	Instruction: <extention mode> "Follower control"
#define I2C_LCD_INST_FOLL 0x60			// ID
#define I2C_LCD_F_ON      0x08			// switch follwer circuit
#define I2C_LCD_F_OFF     0x00			// 
#define I2C_LCD_RAB       0x07			// Rab2-0: V0 generator amplified ratio
#define I2C_LCD_RAB_1     0x00			// x1.0
#define I2C_LCD_RAB_1_25  0x01			// x1.25
#define I2C_LCD_RAB_1_5   0x02			// x1.5
#define I2C_LCD_RAB_1_8   0x03			// x1.8
#define I2C_LCD_RAB_2     0x04			// x2.0
#define I2C_LCD_RAB_2_5   0x05			// x2.5
#define I2C_LCD_RAB_3     0x06			// x3.0
#define I2C_LCD_RAB_3_75  0x07			// x3.75

//	Instruction: <extention mode> "Contrast set"
#define I2C_LCD_INST_CONT 0x70			// ID
#define I2C_LCD_C_LOW     0x0f			// C3~0: Contrast value low nibble

//	Instruction: "Display ON/OFF"
#define I2C_LCD_INST_DISP 0x08			// ID
#define I2C_LCD_DSP_ON    0x04			// Diaplay ON/OFF control
#define I2C_LCD_DSP_OFF   0x00			// 
#define I2C_LCD_CSL_ON    0x02			// Cursor ON/OFF control
#define I2C_LCD_CSL_OFF   0x00			// 
#define I2C_LCD_BLK_ON    0x01			// Cursor blink ON/OFF control
#define I2C_LCD_BLK_OFF   0x00			// 

//	Instruction: "Set DDRAM address"
#define I2C_LCD_INST_DADR 0x80			// ID
#define I2C_LCD_POS_LINE1 0x00			// col=0, line=0
#define I2C_LCD_POS_LINE2 0x40			// col=0, line=1

