//
//  main.c
//
//  Program main / Thermo Hygrometer SDcard logger 0.1
//  Copyright (C) 2009 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.07.03 v0.0  First cording
//	2009.07.06 v0.0a Alpha
//	2009.07.08 v0.0a Append battery status by SMP & link error(auto->static)
//	2009.07.17 v0.0a Append battery shutdown
//	2009.07.18 v0.0b Supported PCB R4, Thermistor=103AT-2 R1=33K
//	2009.07.19 v0.0b bug fix, minus tempture output
//	2009.07.25 v0.1  First release
//

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

#include <limits.h>
//#include <math.h>
#include <stdio.h>
#include <stdlib.h>						// ADD 2009.07.19
#include "i2c_lcd.h"
#include "PSoCGPIOINT.h"

#define PCBR4							// ADD 2009.07.18
#define SAMPLE_COUNT 100				// ADC 1 measure unit
#define MAX(a,b) ((a)>(b)?(a):(b))
#define MIN(a,b) ((a)<(b)?(a):(b))

#ifndef PCBR4							// ADD 2009.07.18
#define R1VALUE  30000					// CHG 2009.07.18 [ohm] Humidity senser series registor
#define NTSA0XV103FE1B0					// ADD 2009.07.18 Thermistor=Murata
#else									// ADD 2009.07.18
#define R1VALUE  33000					// ADD 2009.07.18 [ohm] Humidity senser series registor
#define ISHI103AT						// ADD 2009.07.18 Thermistor=Ishizuka 103AT-2
#endif									// ADD 2009.07.18


//
// Global area
//
#define LOGBUFSZ 10
static char logbuf_yy[LOGBUFSZ];
static char logbuf_mm[LOGBUFSZ];
static char logbuf_dd[LOGBUFSZ];
static char logbuf_hh[LOGBUFSZ];
static char logbuf_mi[LOGBUFSZ];
static char logbuf_ss[LOGBUFSZ];
static int  logbuf_t[LOGBUFSZ];
static int  logbuf_h[LOGBUFSZ];
static char logbufnum = 0;
static int	iv = 1;						// Log interval [sec]
static uchar fp = SDCard_1_MAXFILES;	// SDcard file pointer

//
// Table refrence & Linear interpolation conversion function
//
struct litbl {
	int f;								// return value
	long x;								// reference value
};

#ifdef NTSA0XV103FE1B0					// ADD 2009.07.18
const struct litbl litemp[] = {
	 800,	 501,						// 10xC , A/D count
	 700,	 453,
	 600,	 388,
	 500,	 303,
	 400,	 196,
	 300,	  69,
	 250,	   0,
	 200,	 -72,
	 100,	-215,
	  00,	-345,
	-100,	-451};
#define LITEMPSZ 11
#endif									// ADD 2009.07.18

#ifdef ISHI103AT						// ADD 2009.07.18
const struct litbl litemp[] = {			// ADD 2009.07.18
	 900,	 503,						// ADD 2009.07.18
	 800,	 464,						// ADD 2009.07.18
	 700,	 413,						// ADD 2009.07.18
	 600,	 348,						// ADD 2009.07.18
	 500,	 268,						// ADD 2009.07.18
	 400,	 171,						// ADD 2009.07.18
	 300,	  59,						// ADD 2009.07.18
	 250,	   0,						// ADD 2009.07.18
	 200,	 -62,						// ADD 2009.07.18
	 100,	-186,						// ADD 2009.07.18
	  00,	-302,						// ADD 2009.07.18
	-100,	-403,						// ADD 2009.07.18
	-200,	-483};						// ADD 2009.07.18
#define LITEMPSZ 13						// ADD 2009.07.18
#endif									// ADD 2009.07.18

const struct litbl lihumi[] = {
	 200,	8000000,					// 10x%RH , Ohm
	 300,	1000000,
	 400,	 200000,
	 500,	  60000,
	 600,	  22000,
	 700,	   8300,
	 800,	   3400,
	 900,	   1600,
	1000,	    800};
#define LIHUMISZ 9

int lifunc(long x, struct litbl tbl[], char tblsz) {
	int f;
	char i;

	// Table search
	f = INT_MIN;
	for(i = 0; i < tblsz; i++) {
		if (x > tbl[i].x) {
			if (i == 0) {
				f = INT_MAX;
			// Linear interpolation
			} else {
				long a;
				long b;
				int c;
				int d;

				a = tbl[i-1].x;
				b = tbl[i].x;
				c = tbl[i-1].f;
				d = tbl[i].f;
				f = (x - b) * (c - d) / (a - b) +  d;
			}
			break;
		}
	}
	return(f);
};

//
// 1 sec interval timer ISR
//
#pragma interrupt_handler Timer16_1_ISR
static char ddom[] = {0, 31,28,31,30,31,30,31,31,30,31,30,31};
static char yy =  9;
static char mm =  7;
static char dd =  1;
static char hh =  0;
static char mi =  0;
static char ss =  0;
void interrupt Timer16_1_ISR(void) @ 0x40 {		// DBB20 digital block
	ss++;
	if (ss > 59) {
		ss = 0;
		mi++;
		if (mi > 59) {
			mi = 0;
			hh++;
			if (hh > 23) {
				hh = 0;
				dd++;
				if (((yy % 4) == 0) && ((yy % 100) != 0)) {
					ddom[2] = 29;
				} else {
					ddom[2] = 28;
				}
				if (dd > ddom[mm]) {
					dd = 1;
					mm++;
					if (mm > 12) {
						mm = 1;
						yy++;
					}
				}
			}
		}
	}
	return;
}

//
// Date Time key input & modify
//
void dateset() {
	char y,m,d,h,n;
	char p;

	y = yy;
	m = mm;
	d = dd;
	h = hh;
	n = mi;
	p = 1;
	I2C_LCD_Icon(15, 0x10);		// Clock mark ON
	I2C_LCD_Control(I2C_LCD_CURSOR_BLINK);

	for(;;) {
		// display
		I2C_LCD_Position(0, 0);
		printf("%4d.%02d.%02d %02d:%02d", y+2000, m, d, h, n);
		switch(p) {
		case 1:
			I2C_LCD_Position(0, 3);
			break;
		case 2:
			I2C_LCD_Position(0, 6);
			break;
		case 3:
			I2C_LCD_Position(0, 9);
			break;
		case 4:
			I2C_LCD_Position(0, 12);
			break;
		default:
			I2C_LCD_Position(0, 15);
			break;
		}
		
		// Key off wait
		while (!(Mode_SW_Data_ADDR & Mode_SW_MASK));
		while (!(Incr_SW_Data_ADDR & Incr_SW_MASK));

		// Key push wait
		while ((Mode_SW_Data_ADDR & Mode_SW_MASK) && (Incr_SW_Data_ADDR & Incr_SW_MASK));

		// Key push
		if (!(Mode_SW_Data_ADDR & Mode_SW_MASK)) {
			p++;
			if (p > 5) break;			// forever exit
		}
		if (!(Incr_SW_Data_ADDR & Incr_SW_MASK)) {
			switch(p) {
			case 1:
				y++;
				break;
			case 2:
				m++;
				if (m > 12) {
					m = 1;
				}
				break;
			case 3:
				d++;
				if (d > ddom[m]) {
					d = 1;
				}
				break;
			case 4:
				h++;
				if (h > 23) {
					h = 0;
				}
				break;
			default:
				n++;
				if (n > 59) {
					n = 0;
				}
				break;
			}
		}

		// Chattering cancel
		I2C_LCD_Delay1mTimes(200);		// CHG 2009.07.23  100ms->200ms
	}
	
	// Fix
	yy = y;
	mm = m;
	dd = d;
	hh = h;
	mi = n;
	ss = 0;
	I2C_LCD_Control(I2C_LCD_DISP_ON);
	I2C_LCD_Icon(15, 0);
	
	return;
}

//
// Battery status
//
// ADD 2009.07.08
char getbatt() {
	int b;
	int a;
	int d;
	char s;

	// Get SMP gate open count
	b = Counter16_1_wReadCounter();
	Counter16_1_Start();
	I2C_LCD_Delay1mTimes(1);
	Counter16_1_Stop();
	a = Counter16_1_wReadCounter();
	d = b - a;
//	I2C_LCD_Position(1,0);
//	printf("Pwr=%d ", d);
	if (d < 50) {
		s = 0;							// Full
	} else if (d < 100) {
		s = 1;							// Mid
	} else if (d < 150) {
		s = 2;							// Low
	} else if (d < 225) {				// CHG 2009.07.17
//	} else if (d < 190) {				// CHG 2009.07.17 *debug*
		s = 3;							// Empty
	} else {							// ADD 2009.07.17
		s = 4;							// CHG 2009.07.17  Critical level
	}
	return(s);
}
const char bs2icon[] = {0x1e, 0x1a, 0x12, 0x02, 0x02};

//
// *** M a i n ***
//
void main()
{
	char sw;							// Push switch check counter
	char bs;							// ADD 2009.07.08 Battery status
	char bbs;							// ADD 2009.07.08 Before "bs"

	//
	// Congigure
	//
	Mode_SW_DataShadow |= Mode_SW_MASK;	// Port input mode set
	Mode_SW_Data_ADDR   = Mode_SW_DataShadow;  
	Incr_SW_DataShadow |= Incr_SW_MASK;
	Incr_SW_Data_ADDR   = Incr_SW_DataShadow;  
	SD_CD_DataShadow   |= SD_CD_MASK;		// SDCard module's CD signal pin is pull up 5.4[Kohm] -> bias current=611uA
	SD_CD_Data_ADDR     = SD_CD_DataShadow;	// External pull up registor & myself control

	//
	// init
	//
	bs = getbatt();						// ADD 2009.07.17  Battery check
	if (bs >= 4) {						// ADD 2009.07.17  No power ?
//		VLT_CR |= VLT_CR_SMP;			// ADD 2009.07.17  SMP off
		ABF_CR0 = 0x00;					// ADD 2009.07.17  Analog buffer off
		ARF_CR  = 0x00;					// ADD 2009.07.17  Analog ref off
		for(;;) {						// ADD 2009.07.17
			M8C_Sleep;					// ADD 2009.07.17
		}								// ADD 2009.07.17
	}									// ADD 2009.07.17
	M8C_EnableGInt;
	Timer16_1_Start();					// 1sec interval timer
	Timer16_1_EnableInt();
	SDCard_1_Start();
	Bias_Power_Start();
	LCD_Power_Start();
	I2C_LCD_Start();
	sw  = 0;
	bbs = 99;							// ADD 2009.07.08

	// LCD Hello message
	LCD_Power_Switch(1);
	I2C_LCD_Init();
	printf("ThermoHygroMeter");
	I2C_LCD_Position(1,0);
	printf(" V0.1  2009.7.25");
	{
		int i;
		
		for(i = 0; i < 10; i++) {
			I2C_LCD_Delay1mTimes(200);
		}
	}
	I2C_LCD_Icon(7, 0x00);				// Up/Down mark OFF

	//
	// 1 log/display loop
	//
	for(;;){
		int vt;							// Tempture sensor [A/D count]
		int vspp;						// Humidity sensor + Ri	: Vsource p-p [A/D count]
		int vipp;						// Ri(series register)  : Vi p-p [A/D count]
//		float rs;						// Humidity sensor Z [ohm]
		long rs;						// Humidity sensor Z [ohm]
		int h;							// [%RH]*10
		int t;							// ["C]*10

		//
		// Measure ADC
		//
		{
			int i;
			int v1max, v1min;
			int v2max, v2min;
			int v3sum;

			// PSoC block startup
			Bias_Power_Switch(1);

			PWM8_1_Start();					// 1KHz source
    		Counter8_1_Start();				// BPF clock
			PGA_0_Start(PGA_0_LOWPOWER);	// PWM8->BPF bridge
    		BPF2_1_Start(BPF2_1_LOWPOWER);	// Square wave -> Sin wave
			I2C_LCD_Delay1mTimes(30);		// Stable wait

			PGA_1_Start(PGA_1_MEDPOWER);	// Humidity sensor Vi
    		PGA_2_Start(PGA_2_MEDPOWER);	// Humidity sensor Vsource
    		PGA_3_Start(PGA_3_MEDPOWER);	// Tempture sensor
			TRIADC_1_Start(TRIADC_1_MEDPOWER);

			// ADC sampling
			v1max = v2max = INT_MIN;
			v1min = v2min = INT_MAX;
			v3sum = 0;
			for(i = 0; i < SAMPLE_COUNT; i++){
				int v1, v2, v3;

				TRIADC_1_GetSamples(1);
				while(TRIADC_1_fIsDataAvailable() == 0);
				v1 = TRIADC_1_iGetData1();
				v2 = TRIADC_1_iGetData2();
				v3 = TRIADC_1_iGetData3ClearFlag();
				v1max = MAX(v1max, v1);
				v1min = MIN(v1min, v1);
				v2max = MAX(v2max, v2);
				v2min = MIN(v2min, v2);
				v3sum += v3;
			}
			vipp = v1max - v1min;
			vspp = v2max - v2min;
			vt   = v3sum / SAMPLE_COUNT;
			
			// Power Consumption save
			TRIADC_1_Stop();
    		PGA_3_Stop();
    		PGA_2_Stop();
    		PGA_1_Stop();
   			BPF2_1_Stop();
    		PGA_0_Stop();
			Counter8_1_Stop();
    		PWM8_1_Stop();
			Bias_Power_Switch(0);
		}

		//
		// Calcurate tempture
		//
		t = lifunc(vt, litemp, LITEMPSZ);
		t = max(t, -999);				// ADD 2009.07.19
		t = min(t,  999);				// ADD 2009.07.19

		//
		// Calcutate relative humidity
		//
		if (vipp != 0) {
//			rs = (float)R1VALUE * (float)vspp / (float)vipp - (float)R1VALUE;	// more memory
//			h = 338.2 * pow(rs, -0.1765);										// more memory
			rs = (long)R1VALUE * (long)vspp / vipp - R1VALUE;
			h = lifunc(rs, lihumi, LIHUMISZ);	// %RH
			h += ((250 - t) * 67 / 100);		// Temp adjust 1.0"C:-0.67%
			h = max(h,   0);					// ADD 2009.07.19
			h = min(h, 999);					// ADD 2009.07.19
		}

		//
		// Display refresh
		//
		I2C_LCD_Position(0,0);
		printf("%3d.%d\362C %2d.%d%%RH ",		// UPD 2009.07.19
				t/10, abs(t)%10, h/10, h%10);	// UPD 2009.07.19
		I2C_LCD_Position(1,0);
		printf("%4d.%2d.%2d %2d:%02d", yy+2000, mm, dd, hh, mi);

		//
		// SDcard write
		//
		if (fp != SDCard_1_MAXFILES) {	// file active
			
			// Buffer entry
			logbuf_yy[logbufnum] = yy;
			logbuf_mm[logbufnum] = mm;
			logbuf_dd[logbufnum] = dd;
			logbuf_hh[logbufnum] = hh;
			logbuf_mi[logbufnum] = mi;
			logbuf_ss[logbufnum] = ss;
			logbuf_t[logbufnum]  = t;
			logbuf_h[logbufnum]  = h;
			logbufnum++;

			// Buffer full, physical write
			if ((logbufnum >= LOGBUFSZ) || (iv >= 60)) {
				int i;

				I2C_LCD_Icon(9, 0x10);	// Lock icon on
				
				for(i = 0; i < logbufnum; i++) {
					static char buf[40];												// UPD 2009.07.19  static->auto
					
					sprintf(buf, "\"%4d/%02d/%02d %02d:%02d:%02d\",%d.%d,%d.%d\r\n",	// UPD 2009.07.19
						2000+logbuf_yy[i], logbuf_mm[i], logbuf_dd[i],
						logbuf_hh[i], logbuf_mi[i], logbuf_ss[i],
						logbuf_t[i]/10, abs(logbuf_t[i])%10,							// UPD 2009.07.19
						logbuf_h[i]/10, logbuf_h[i]%10);
					SDCard_1_fputs(buf, fp);
				}
				SDCard_1_fflush(fp);
				logbufnum = 0;
				
				I2C_LCD_Icon(9, 0);		// Lock icon off
			}
		}


		//
		// Get battery power
		//
		bs = getbatt();					// ADD 2009.07.08   After peak power


		//
		// Wait next log
		//
		for(;;) {
			M8C_ClearWDTAndSleep;

			// Battery icon update/blink
			if (bs < 4) {							// ADD 2009.07.08  Normal level
				if (bbs != bs) {					// ADD 2009.07.08  Status change
					I2C_LCD_Icon(13, bs2icon[bs]);	// ADD 2009.07.08  status lavel -> icon patturn
				}									// ADD 2009.07.08
			} else {								// ADD 2009.07.08  Critical level
				if (ss & 1) {						// ADD 2009.07.08  Blink odd second
					I2C_LCD_Icon(13, 0);			// ADD 2009.07.08
				} else {							// ADD 2009.07.08
					I2C_LCD_Icon(13, bs2icon[bs]);	// ADD 2009.07.08
				}									// ADD 2009.07.08
			}										// ADD 2009.07.08
			bbs = bs;								// ADD 2009.07.08

			// Push switch scan
			if (!(Mode_SW_Data_ADDR & Mode_SW_MASK)) {
				sw++;
				if (sw > 3) {
					dateset();
				}
			} else {
				sw = 0;
			}
			if (!(Incr_SW_Data_ADDR & Incr_SW_MASK)) {
				switch(iv) {
				case 1:
					iv = 10;
					I2C_LCD_Icon(7, 0x10);	// Up ON
					break;
				case 10:
					iv = 60;
					I2C_LCD_Icon(7, 0x08);	// Down ON
					break;
				case 60:
					iv = 600;
					I2C_LCD_Icon(7, 0x18);	// Up/Down ON
					break;
				default:
					iv = 1;
					I2C_LCD_Icon(7, 0x00);	// Up down OFF
					break;
				}
			}
			
			// SDcard detect
			if (!(SD_CD_Data_ADDR & SD_CD_MASK)) {

				// Unopen
				if (fp == SDCard_1_MAXFILES) {
					uchar s;
					
					// File open
					I2C_LCD_Position(1,0);
					printf("SDcard inserted ");
					SDCard_1_Select(SDCard_1_ENABLE);
					s = SDCard_1_InitCard();
					if ((s == 0x12) || (s == 0x42)) {
						fp = SDCard_1_fopen("TH.csv","a");
						if (fp != SDCard_1_MAXFILES) {
							SDCard_1_fputs("TimeStamp,Temp[C],Humi[%RH]\r\n", fp);
							SDCard_1_fflush(fp);

							I2C_LCD_Icon(6, 0x10);		// ArrowBox ON
							I2C_LCD_Position(1,0);
							printf("Logging start   ");
						} else {
							I2C_LCD_Position(1,0);
							printf("SDcard open err ");
						}
					} else {
						I2C_LCD_Position(1,0);
						printf("SDcard FAT32? %02x", s);
					}
				}
	
			// SDcard eject
			} else {
				if (fp != SDCard_1_MAXFILES) {	// file active
					SDCard_1_fclose(fp);
					SDCard_1_Select(SDCard_1_DISABLE);
					fp = SDCard_1_MAXFILES;
					
					I2C_LCD_Icon(6, 0x00);
				}
			}

			// Sleep until 1sec intrrupt
			{
				BYTE ss;				// status save
				BYTE sa;				//
		
				sa = ABF_CR0;			// Analog Output Buffer Control Register 0
				ss = ARF_CR;			// Analog Reference Control Register
				
				ABF_CR0 = 0x00;			// block power off (~-230uA)
				ARF_CR  = 0x00;			// block power off (~-3mA)
				M8C_Sleep;				// clock stop      (~3uA)
				
				ARF_CR  = ss;			// dummy
				ARF_CR  = ss;			// power resume
				ABF_CR0 = sa;			// 
			}

			// Next log timing check
			if  (iv ==   1)										break;
			if ((iv ==  10) && ((ss % 10) == 0))				break;
			if ((iv ==  60) && (ss == 0))						break;
			if ((iv == 600) && (ss == 0) && ((mi % 10) == 0))	break;
		}

	}

// 	Counter8_2_Stop();
//	I2C_LCD_Stop();
//	SDCard_1_Stop();
//	Timer16_1_Stop();
}
